#! /bin/bash
#
# KMI startup for Linux, v1.20

################################################################################

ErrorExit()
{
    echo -e "\nError: $1\n"
    exit 1
}

################################################################################

APP_HOME=/usr/rock/SMARTSDist

if [ ! -d "$APP_HOME" ]; then
    APP_HOME=/opt/rockhound/SMARTSDist
fi

APP_NAME="Rockhound"
ORIG_SNAME="linuxrun.sh"
RERUN_SNAME="tmp_$ORIG_SNAME"
CPATH=""
SELF_NAME=`basename $0`
BF_RELOAD_THRESHOLD_SECS=900
AVOIDDSPRELOAD=0
LBF=""
QNBF=""

export LD_LIBRARY_PATH=$APP_HOME/lib:$LD_LIBRARY_PATH
umask 002

################################################################################
# Become a RERUN copy ASAP

if [ "$SELF_NAME" != "$RERUN_SNAME" ]; then
    cp "$APP_HOME/bin/$ORIG_SNAME" "$APP_HOME/bin/$RERUN_SNAME" || ErrorExit "Unable to write $APP_HOME/bin/$RERUN_SNAME\nPossible permission/wrong user problem."
    exec "$APP_HOME/bin/$RERUN_SNAME" "$1"
    exit 0
fi

# At this point we should always be running as a RERUN copy

################################################################################

IfExistCopy()
{
    if [ -f "$1" ]; then
        cp "$1" "$2" 
        [ $? != 0 ] && ErrorExit "Unable to copy $1 to $2"
    fi
}

###############################################################################

IfExistDel()
{
    if [ -f "$1" ]; then
        rm "$1" 
        [ $? != 0 ] && ErrorExit "Unable to delete $1"
    fi
}

################################################################################

LaunchRHAutoRun()
{
    if [ -x "$APP_HOME/bin/rhautorun.sh" ]; then

        ps -e | grep -q rhautorun.sh

        if [ $? == 0 ]; then
            echo "rhautorun.sh is already running."
            return
        fi

        echo "Launching rhautorun.sh..."
        "$APP_HOME/bin/rhautorun.sh" &
    fi
}

################################################################################

ManageInjar()
{
    if [ -f "$APP_HOME/injar/update.jar" ]; then
        echo "New update found, unpacking..."
        IfExistCopy "$APP_HOME/update.jar" "$APP_HOME/update.jar.prev"
        IfExistCopy "$APP_HOME/injar/update.jar" "$APP_HOME/update.jar"
        IfExistDel "$APP_HOME/injar/update.jar"
        IfExistDel "$APP_HOME/done.tag"
        cd "$APP_HOME/.."
        jar xvf "$APP_HOME/update.jar"
        cd "$APP_HOME"
        LBF=`which loadbf 2>/dev/null`
        [ "$LBF" != "" ] && $LBF -u
        sync;sync


        if [ ! -f "$APP_HOME/done.tag" ]; then
            [ ! -f "$APP_HOME/update.jar.prev" ] && ErrorExit "New update failed, no previous file to restore."
            echo "New update failed, restoring previous file..."
            IfExistCopy "$APP_HOME/update.jar.prev" "$APP_HOME/update.jar"
            cd "$APP_HOME/.."
            jar xvf "$APP_HOME/update.jar"
            cd "$APP_HOME"
            sync;sync
            [ ! -f "$APP_HOME/done.tag" ] && ErrorExit "Update restoration failed."
        fi

        exec "$APP_HOME/bin/$ORIG_SNAME"
        exit 0
    fi

    if [ -f "$APP_HOME/injar/config.jar" ]; then
        echo "New config found, unpacking..."
        IfExistCopy "$APP_HOME/config/config.jar" "$APP_HOME/config/config.jar.prev"
        IfExistCopy "$APP_HOME/injar/config.jar" "$APP_HOME/config/config.jar"
        IfExistDel "$APP_HOME/injar/config.jar"
        IfExistDel "$APP_HOME/config/done.tag"
        rm "$APP_HOME/config/modules/*.cfg" > /dev/null 2>&1
        cd "$APP_HOME/config"
        jar xvf "$APP_HOME/config/config.jar"
        cd "$APP_HOME"
        sync;sync

        if [ ! -f "$APP_HOME/config/done.tag" ]; then
            [ ! -f "$APP_HOME/config/config.jar.prev" ] && ErrorExit "New config failed, no previous file to restore."
            echo "New config failed, restoring previous file..."
            IfExistCopy "$APP_HOME/config/config.jar.prev" "$APP_HOME/config/config.jar"
            cd "$APP_HOME/config"
            jar xvf "$APP_HOME/update.jar"
            cd "$APP_HOME"
            sync;sync
            [ ! -f "$APP_HOME/config/done.tag" ] && ErrorExit "Config restoration failed."
        fi
    fi
}

################################################################################

ManageArmLibs()
{
    grep -iq XScale-PXA255 /proc/cpuinfo 2> /dev/null

    if [ $? == 0 ]; then
        if [ -f $APP_HOME/lib/libkmijsupportarmlinux-OABI.so ]; then
            echo "Setting up OABI shared libs..."
            cp $APP_HOME/lib/libkmijsupportarmlinux-OABI.so $APP_HOME/lib/libkmijsupportarmlinux.so
            cp $APP_HOME/lib/libkmijsupportarmlinux-OABI.so $APP_HOME/lib/libkmijsupportarmlinux-OABI.so.installed
            rm $APP_HOME/lib/libkmijsupportarmlinux-OABI.so
            fi
    else
            if [ -f $APP_HOME/lib/libkmijsupportarmlinux-EABI.so ]; then
                echo "Setting up EABI shared libs..."
                cp $APP_HOME/lib/libkmijsupportarmlinux-EABI.so $APP_HOME/lib/libkmijsupportarmlinux.so
                cp $APP_HOME/lib/libkmijsupportarmlinux-EABI.so $APP_HOME/lib/libkmijsupportarmlinux-EABI.so.installed
                rm $APP_HOME/lib/libkmijsupportarmlinux-EABI.so
            fi
    fi
}


################################################################################
# Dynamically create the APP classpath and place in CPATH

LoadCPATH()
{
    local len
    local count

    local -a app_jars=( `ls -1 $APP_HOME/lib/*.jar` )

    len=${#app_jars[*]}
    CPATH="$APP_HOME/classes"
    count=0

    while [ "$count" -lt "$len" ]
    do
        CPATH=$CPATH:${app_jars[$count]}
        count=$(($count+1))
    done

    #DEBUG: echo "ClassPath: $CPATH"
}

################################################################################

sleep 1
clear

cd "$APP_HOME"
chmod 755 $APP_HOME/bin/* 2> /dev/null
export PATH="$PATH:$APP_HOME/bin"

IfExistDel "$APP_HOME/exit.cfg"
LBF=`which loadbf 2>/dev/null`
QNBF=`which nbfboards 2>/dev/null`

echo ""
echo -n "KMI $APP_NAME Software (www.kmi.com)"

[ "$1" == "INIT" ] && [ "$LBF" != "" ] && $LBF -u

sleep 1
echo ""
ManageArmLibs

echo "Checking KMI $APP_NAME configuration..."

if [ ! -f "$APP_HOME/config/config.jar" ]; then
    if [ -f "$APP_HOME/save/user.jar" ]; then
        echo "No config found, copying saved user config..."
        cp "$APP_HOME/save/user.jar" "$APP_HOME/injar/config.jar" || ErrorExit "Missing saved user config."
    else
        echo "No config found, copying basic communication config..."
        cp "$APP_HOME/config/consoleonly.jar" "$APP_HOME/injar/config.jar" || ErrorExit "Missing basic communication config."
    fi
fi

ManageInjar
LoadCPATH

if [ "$LBF" != "" ]; then

     while (true)
     do
         $LBF -a
         if [ $? != 4 ]; then
             break
         fi

         $LBF -u
         sleep 2
     done
fi

[ "$NBFBOARDS" == "" ] && NBFBOARDS="0"
[ "$QNBF" != "" ] && [ "$NBFBOARDS" == "0" ] && NBFBOARDS=`$QNBF`

set -o pipefail
mkdir /tmp/kmi > /dev/null 2>&1
LaunchRHAutoRun
echo "Starting KMI $APP_NAME..."

while true
do
    sleep 2
    TSTART=`date +%s`
    java -DNBFBOARDS="$NBFBOARDS" -DAVOIDDSPRELOAD="$AVOIDDSPRELOAD" -classpath "$CPATH" kmi.smarts.app.SMARTSApp 3>&1 1>&2 2>&3 | tee -a SMARTSError.txt
    ECODE=$?
    TEND=`date +%s`

    if [ -f "$APP_HOME/exit.cfg" ]; then
         echo "Stopped KMI $APP_NAME application."
         exit 0
    fi

    echo "Restarting KMI $APP_NAME..." > /dev/console

    if [ $ECODE != 0 ] && [ "$LBF" != "" ]; then
        ELAPSED=$(( TEND - TSTART ))
        if (( $ELAPSED < $BF_RELOAD_THRESHOLD_SECS )); then
            $LBF -u
            echo "DSP firmware will be reloaded."
        else
            AVOIDDSPRELOAD=1
            echo "Avoiding DSP firmware reload."
            continue
        fi
    fi

    break
done

exec "$APP_HOME/bin/$ORIG_SNAME"

